import { ButtonProps } from 'react-bootstrap';
import { Color } from 'react-bootstrap/esm/types';
import { Component } from 'react';
import createParser from 'editorjs-html';
import { CSSProperties } from 'react';
import { Defer } from 'iterable-observer';
import { DetailedHTMLProps } from 'react';
import { DropdownButtonProps } from 'react-bootstrap';
import { FC } from 'react';
import { HTMLAttributes } from 'react';
import { ImgHTMLAttributes } from 'react';
import { InputHTMLAttributes } from 'react';
import { JSX as JSX_2 } from 'react/jsx-runtime';
import { NavProps } from 'react-bootstrap';
import { OutputData } from '@editorjs/editorjs';
import { OverlayTriggerProps } from 'react-bootstrap';
import { PropsWithChildren } from 'react';
import { PropsWithoutRef } from 'react';
import { PropsWithRef } from 'react';
import { PureComponent } from 'react';
import { ReactNode } from 'react';
import { RefCallback } from 'react';
import { RefObject } from 'react';
import { SpinnerProps } from 'react-bootstrap';
import { TableProps } from 'react-bootstrap';
import { TimeData } from 'web-utility';
import { WrapperProps } from '@react-editor-js/core';

export declare function animate(name: string): RefCallback<Element>;

export declare const Avatar: FC<AvatarProps>;

export declare interface AvatarProps extends ImgHTMLAttributes<HTMLImageElement> {
    size?: number;
}

export declare class ClickBoundary extends PureComponent<ClickBoundaryProps> {
    root: RefObject<HTMLDivElement>;
    switchClick: (event: MouseEvent) => void;
    componentDidMount(): void;
    componentWillUnmount(): void;
    render(): JSX_2.Element;
}

export declare type ClickBoundaryProps = PropsWithChildren<{
    onInnerClick?: (event: MouseEvent) => any;
    onOuterClick?: (event: MouseEvent) => any;
} & DetailedHTMLProps<HTMLAttributes<HTMLDivElement>, HTMLDivElement>>;

export declare function CodeBlock({ language, children }: CodeBlockProps): JSX_2.Element;

export declare type CodeBlockProps = PropsWithChildren<{
    language: string;
}>;

export declare interface DateData {
    date: TimeData;
    content: ReactNode;
    link?: string;
}

export declare class Dialog<T = any> {
    accessor defer: Defer<T> | undefined;
    Component: FC;
    constructor(Factory: FC<DialogProps<T>>);
    open(): Promise<T>;
}

export declare class DialogClose extends Error {
    constructor(message?: string);
}

export declare interface DialogProps<T = any> {
    defer?: Defer<T>;
}

export declare class Editor extends PureComponent<EditorProps> {
    static displayName: string;
    private core?;
    accessor innerValue: OutputData;
    save: () => Promise<OutputData>;
    render(): JSX_2.Element;
}

export declare const EditorHTML: FC<EditorHTMLProps>;

export declare type EditorHTMLProps = PropsWithoutRef<{
    className?: string;
    data: OutputData;
}>;

export declare type EditorProps = PropsWithoutRef<WrapperProps & Pick<InputHTMLAttributes<HTMLInputElement>, 'name' | 'required'>>;

declare type HeadingMeta = Record<'id' | 'text', string> & Record<'level' | 'top', number>;

export declare const Icon: FC<IconProps>;

export declare type IconProps = PropsWithRef<HTMLAttributes<HTMLSpanElement> & {
    name: string;
    size?: number;
}>;

export declare const Loading: FC<LoadingProps>;

export declare type LoadingProps = PropsWithChildren<{}>;

/**
 * Re-implement from https://github.com/EasyWebApp/BootCell/blob/3d30027a97fe0a8c4ab8fabc8dfef22aede04de7/source/Calendar/MonthCalendar.tsx
 */
export declare class MonthCalendar extends Component<MonthCalendarProps> {
    static displayName: string;
    observedProps: MonthCalendarProps;
    get weekFormatter(): Intl.DateTimeFormat;
    accessor currentDate: Date;
    get dateGrid(): Date[][];
    changeMonth(delta: number): void;
    renderDate: (date: Date) => JSX_2.Element;
    render(): JSX_2.Element;
}

export declare interface MonthCalendarProps extends Omit<TableProps, 'onChange' | 'onSelect'>, Pick<ButtonProps, 'variant'> {
    locale?: Navigator['language'];
    value?: DateData[];
    onSelect?: (value: DateData) => any;
    onChange?: (date: Date) => any;
}

export declare const Nameplate: FC<NameplateProps>;

export declare type NameplateProps = PropsWithoutRef<{
    avatar: string;
    name: string;
}>;

declare const Option_2: FC<OptionProps>;
export { Option_2 as Option }

export declare type OptionProps = PropsWithChildren<{
    className?: string;
    style?: CSSProperties;
    value?: string;
}>;

/**
 * ToDo: {@link https://react-bootstrap.github.io/docs/components/overlays#disabled-elements}
 */
export declare const OverlayBox: FC<OverlayBoxProps>;

export declare interface OverlayBoxProps extends Omit<OverlayTriggerProps, 'overlay'> {
    title: ReactNode;
    detail?: ReactNode;
}

export declare class PageNav extends PureComponent<PageNavProps> {
    static displayName: string;
    accessor list: HeadingMeta[];
    accessor scrollY: number;
    get currentActiveId(): string;
    updateScrollY: () => number;
    componentDidMount(): void;
    componentWillUnmount(): void;
    scrollTo: (meta: HeadingMeta) => () => void;
    renderLink: ({ id, level, text, top }: HeadingMeta) => JSX_2.Element;
    render(): JSX_2.Element;
}

export declare interface PageNavProps extends NavProps {
    depth?: number;
    onItemClick?: (item: HeadingMeta) => any;
}

export declare const parser: createParser;

export declare const Select: FC<SelectProps>;

export declare interface SelectProps extends OptionProps, Pick<DropdownButtonProps, 'variant' | 'menuVariant'> {
    onChange?: (value: string) => any;
    valueRender?: (value: string) => ReactNode;
}

export declare const SpinnerButton: FC<SpinnerButtonProps>;

export declare interface SpinnerButtonProps extends ButtonProps, Pick<SpinnerProps, 'animation'> {
    loading?: boolean;
}

export declare const TableSpinner: FC<TableSpinnerProps>;

export declare type TableSpinnerProps = PropsWithoutRef<{
    colSpan: number;
}>;

export declare function text2color(raw: string, excludeColors?: VariantColor[]): string;

export declare const TimeDistance: FC<TimeDistanceProps>;

export declare type TimeDistanceProps = PropsWithoutRef<{
    className?: string;
    date: TimeData;
    unitWords?: Record<string, string>;
    beforeWord?: string;
    afterWord?: string;
}>;

export declare class TypeEcho extends PureComponent<TypeEchoProps> {
    static displayName: string;
    accessor echoed: string;
    init: RefCallback<HTMLPreElement>;
    render(): JSX_2.Element;
}

export declare type TypeEchoProps = PropsWithoutRef<{
    className?: string;
    text: string;
    intervals?: number;
}>;

export declare type VariantColor = Exclude<Color, 'white' | 'muted'>;

export declare const VariantColors: VariantColor[];

export { }
